import display
import status
import customsprites
import projectile
import mob
from helper import *

class TType:
	[Wall, Trap, Turret] = range(3)

class Terrain:
	def __init__(self):
		self.sprite = customsprites.TerrainSprite('BrickColumn')
		self.type = TType.Wall
		self.sprite.render = customsprites.Render.Tall
		self.path_weight = 100
		self.elevated = True
		self.position = None
		self.vitality = 400
		self.damage = 0
		self.attack = 0
		self.sprite.cycle = 10
		self.sprite.speed = 0
		self.vision = 10		
		self.cost = 15
		self.active_type = False
		self.name = "Test Terrain"
	
	def update(self, tick):
		if self.damage >= self.vitality:
			self.smashed()
		
		if self.active_type and self.sprite.idle:
			self.work_it()
	
	def work_it(self):
		pass
	
	def smashed(self):
		display.get_map().del_terrain(self)
	
	def set_position(self, t_pos):
		self.position = tup2rect(t_pos)

	def clicked(self):
		pass
	
	def stepped_on(self, mob):
		pass


class HedgeWall(Terrain):
	def __init__(self):
		Terrain.__init__(self)
		self.sprite = customsprites.TerrainSprite('Hedge')
		self.type = TType.Wall
		self.sprite.render = customsprites.Render.Tall
		self.path_weight = 100
		self.elevated = False
		self.vitality = 100
		self.cost = 2
		self.name = "Hedge Wall"

class BrickWall(Terrain):
	def __init__(self):
		Terrain.__init__(self)
		self.sprite = customsprites.TerrainSprite('BrickColumn')
		self.type = TType.Wall
		self.sprite.render = customsprites.Render.Tall
		self.path_weight = 150
		self.elevated = True
		self.vitality = 400
		self.cost = 10
		self.name = "Brick Wall"

class PitTrap(Terrain):
	def __init__(self):
		Terrain.__init__(self)
		self.sprite = customsprites.TerrainSprite('HiddenPit')
		self.type = TType.Trap
		self.sprite.render = customsprites.Render.Flat
		self.path_weight = 0
		self.elevated = False
		self.vitality = 5
		self.cost = 30
		self.name = "Pit Trap"

	def stepped_on(self, m):
		if m.enemy and not m.mood == mob.Mood.Flee and self.path_weight == 0:
			self.vitality -= 1
			if self.vitality == 0:
				display.get_map().tiles[self.position[0]][self.position[1]] = 1
				display.get_map().render_tile(self.position)
				display.get_map().weights[self.position[0]][self.position[1]] = 1
			else:
				m.pitted()
				self.path_weight = 100
				display.get_map().tiles[self.position[0]][self.position[1]] = 4
				display.get_map().render_tile(self.position)
				display.get_map().weights[self.position[0]][self.position[1]] = 200
				self.sprite.change_image('OpenPit')
	
	def clicked(self):
		if self.path_weight > 0:
			for m in display.get_map().mobs:
				if m.position == self.position:
					return
			display.get_map().tiles[self.position[0]][self.position[1]] = 1
			display.get_map().render_tile(self.position)
			if self.vitality == 0:
				# go away
				pass
			else:
				self.sprite.change_image('HiddenPit')
				self.path_weight = 0
				display.get_map().tiles[self.position[0]][self.position[1]] = 1
				display.get_map().render_tile(self.position)
				display.get_map().weights[self.position[0]][self.position[1]] = 1

class ManaTrap(Terrain):
	def __init__(self):
		Terrain.__init__(self)
		self.sprite = customsprites.TerrainSprite('ManaTrap')
		self.type = TType.Trap
		self.sprite.render = customsprites.Render.Flat
		self.path_weight = 0
		self.elevated = False
		self.vitality = 8
		self.cost = 20
		self.delay = 0
		self.active_type = True
		self.name = "Mana Glyph"

	def stepped_on(self, m):
		if m.enemy and not m.mood == mob.Mood.Flee:
			status.alter_mana(15)
			self.sprite.change_image('ManaTrapTriggered')
			self.delay = 2
			self.vitality -= 1
	
	def work_it(self):
		if self.delay > 0:
			self.delay -= 1
			if self.delay == 0:
				self.sprite.change_image('ManaTrap')


class FlameTower(Terrain):
	def __init__(self):
		Terrain.__init__(self)
		self.sprite = customsprites.TerrainSprite('FlameTower')
		self.type = TType.Turret
		self.sprite.render = customsprites.Render.Tall
		self.path_weight = 200
		self.elevated = False
		self.vitality = 100
		self.cost = 300
		self.cycle = 15
		self.delay = 0
		self.vision = 20
		self.active_type = True
		self.name = "Flame Tower"

	def work_it(self):
		self.delay += 1
		if self.delay >= self.cycle:
			m = self.enemy_visible()
			if m != None:
				display.get_map().add_projectile(projectile.Fireball(m, 40), self.position)
			self.delay = 0
	
	def enemy_visible(self):
		for m in display.get_map().mobs:
			if m.enemy and self.dist_to(m) < self.vision and m.damage < m.vitality and m.mood != mob.Mood.Flee:
				return m
		return None
	
	def dist_to(self, mob):
		return abs(self.position.left - mob.position.left) + abs(self.position.top - mob.position.top)





